import React from "react";
import ReactMapGL from "react-map-gl";
import {
  Marker,
  NavigationControl,
  FullscreenControl,
  ScaleControl,
  GeolocateControl,
  Layer,
  Source,
} from "react-map-gl";
import "./../css/maps.css";
import mapboxToken from "./../config/mapboxToken.js";
import {calcDistance} from "./../helpers/distance";
import {longitude, latitude, mapWidth, mapHeight, markerIcon} from "./../config/variables";
import {Button} from "reactstrap";
import {getRoute} from "./../helpers/mapboxHelper";
import PathButtons from "./../PathButtons";

export default class MapGL extends React.Component{
  constructor(props){
    super(props);
    this.state = {
      viewport: {
        width: mapWidth,
        height: mapHeight,
        latitude: latitude,
        longitude: longitude,
        zoom: 15,
        bearing: 0,
        pitch: 0
      },
      path: [[longitude, latitude]],
      distance: 0,
    }
    this.getRoute = getRoute.bind(this)
  }

  updateState = (key, value) => {
    this.setState({[key]: value})
    this.getDistance()
  }

  addMarker = event => {
    const lng = event.lngLat[0]
    const lat = event.lngLat[1]
    this.getRoute(lng, lat);
  }

  updateMarker = (idx, event) => {
    const lng = event.lngLat[0]
    const lat = event.lngLat[1]
    let pathCopy = this.state.path.slice();
    pathCopy[idx] = [lng, lat]
    this.setState({path: pathCopy})
    this.getDistance()
  }

  getDistance = () => {
    let dst = calcDistance(this.state.path)
    this.setState({distance: dst})
  }

  clearPath = () => {
    this.setState({path: [], distance: 0})
  }

  render() {
    return (
      <div>
        <div className="map">
          <ReactMapGL
            {...this.state.viewport}
            onViewportChange={(viewport) => this.setState({viewport})}
            mapboxApiAccessToken={mapboxToken}
            mapStyle="mapbox://styles/mapbox/streets-v11"
            onClick={this.addMarker}
          >

          <Source
            id="routeSource"
            type="geojson"
            data={{type: "LineString", coordinates: this.state.path}}
          >
            <Layer
              source="routeSource"
              id="route"
              type="line"
              paint={{"line-color": "#888", "line-width": 3}}
              layout= {{"line-join": "round", "line-cap": "round"}}
            />
          </Source>

          {this.state.path.map((pos, idx) => {
            return(
            <Marker
              longitude={pos[0]}
              latitude={pos[1]}
              captureClick={false}
              draggable={true}
              offsetTop={-20}
              offsetLeft={-8}
              key={idx}
              onDragEnd={(e) => this.updateMarker(idx, e)}
            >
              <svg height={20} viewBox="0 0 24 24" style={{fill: "#343a40", stroke: "none", position: "absolute", top: "3px"}}>
                {(idx===0 || idx===this.state.path.length-1 ?
                  (<path d={markerIcon} /> )
                  : (<circle cx="10" cy="19" r="5"/>)
                )}
              </svg>
            </Marker>
          )})}

            <div className="controllers">
              <GeolocateControl />
              <FullscreenControl />
              <NavigationControl />
              <ScaleControl />
              <small> {'\u00A9'} Mapbox {'\u00A9'} OpenStreetMap </small>
            </div>
          </ReactMapGL>
        </div>

        <div className="mapInfo">
          <Button onClick={this.clearPath}>Clear path</Button>
          <Button onClick={this.getDistance}>Calculate Distance</Button>
          <p>
            Distanz: {Math.round(this.state.distance*100)/100} km
          </p>
          <PathButtons updateState={this.updateState} path={this.state.path} switchLatLng={true}/>
        </div>
      </div>
    );
  }
}